import * as d3 from 'd3';

let PluseIDCount = 0;

export class PulseLine {
    constructor() {
    }

    loadData(pathString, id) {
        this.path = pathString;
        this.pluseId = id;
    }

    /**
     * 
     * @param {d3.Selection} parentSelection 
     */
    loadToSelection(parentSelection, config) {
        config = config || {};
        const pluse = parentSelection.append('g');
        
        const gId = 'pluse-g' + this.pluseId;
        const pId = 'pluse-p' + this.pluseId;
        const mId = 'pluse-m' + this.pluseId;
        const uId = 'pluse-u' + this.pluseId;

        pluse.append('radialGradient')
            .attr("id", gId)
            .selectAll("stop")
            .data([
                { offset: "0%", color: "white" },
                { offset: "25%", color: "#CB81DA" },
                { offset: "95%", color: "rgb(70, 70, 222)", opacity: 0 }
            ])
            .enter()
            .append("stop")
            .attr("offset", d => d.offset)
            .attr("stop-color", d => d.color)
            .attr("stop-opacity", d => d.opacity || 1);

        pluse.append("path")
            .attr("id", pId)
            .attr('fill', 'none')
            .attr("d", "M133,613 L470,613 L470,515.5 L661,515.5");
        
        const mask = pluse.append("mask")
            .attr("id", mId);

        mask.append("use")
            .attr("id", uId)
            .attr("href", pId)
            .attr("stroke", "white")
            .attr("stroke-width", 5)
            .attr("fill", "none")
            .attr("stroke-dasharray", "100 200")
            .attr("stroke-linecap", "round");
    
        // 创建使用 mask 和 radialGradient 的 <g>
        const maskedG = pluse.append("g")
            .attr("mask", `url(#${mId})`);
    
        // 创建 circle 和 animateMotion
        maskedG.append("circle")
            .attr("r", 50)
            .attr("fill", `url(#${pId})`)
            .append("animateMotion")
            .attr("dur", "4500ms")
            .attr("repeatCount", "indefinite")
            .append("mpath")
            .attr("href", `#${pId}`);
    }

    animation() {
        const pId = 'pluse-p' + this.pluseId;
        const uId = 'pluse-u' + this.pluseId;

        const pathElement = document.getElementById(pId);
        const pluseElement = document.getElementById(uId);

        const pathLength = pathElement.getTotalLength();

        const keyframes = new KeyframeEffect(
            pluseElement, [{
                strokeDasharray: `100 ${pathLength}`,
                strokeDashoffset: "100"
            },
            {
                strokeDasharray: `100 ${pathLength}`,
                strokeDashoffset: `-${pathLength - 100}`
            }
        ], {
            duration: this.duration,
            iterations: Infinity
        }
        );

        const pulsationAnimation = new Animation(keyframes, document.timeline);
        return pulsationAnimation;
    }
}

/**
 * @typedef PulseLineSvgConfig
 * @property {string} pluseStartColor
 * @property {string} pluseMiddleColor
 * @property {string} pluseEndColor
 * @property {string} transform
 * @property {number} duration
 */